@@ -40,19 +40,24 @@ class AbstractBrowser:
 
         self.scan_mode = "all"
         self.history = [os.getcwd()]
-        self.key_list = ["-FILES_BROWSER-","-DIR-","-DIRECTORY_BROWSER-","-FILES_LIST-","-SCAN_MODE_ALL-","-SELECT_HIGHLIGHTED-","-SCAN-", "-SELECT_HIGHLIGHTED-", "-MODE-", "-BROWSE_BACKWARDS-", "-BROWSE_FORWARDS-"]
+        self.key_list = ['-BROWSER_LIST-','-CLEAR_LIST-','-ADD_TO_LIST-',"-FILES_BROWSER-","-DIR-","-DIRECTORY_BROWSER-","-FILES_LIST-","-SCAN_MODE_ALL-","-SELECT_HIGHLIGHTED-","-SCAN-", "-SELECT_HIGHLIGHTED-", "-MODE-", "-BROWSE_BACKWARDS-", "-BROWSE_FORWARDS-"]
 
     def handle_event(self,values,event,window):
+          
         self.event,self.values,self.window=event,values,window
-        self.event_key_js = get_event_key_js(self.event,key_list=self.key_list)
+        if event not in self.key_list:
+            try:
+                self.event_key_js = get_event_key_js(self.event,key_list=self.key_list)
+            except:
+                self.event_key_js={"found":event,"section":'',"event":event}
         if self.event_key_js['found']:
             return self.while_static(self.window, self.event_key_js, self.values)
         else:
             return self.event, self.values
     def scan_it(self,directory):
-        self.last_scan=os.path.isfile(self.values[self.event_key_js['-DIR-']])
-        self.scan_results = self.scan_directory(directory, self.scan_mode)
-        self.browse_update(key=self.event_key_js['-FILES_LIST-'],args={"values":self.scan_results})
+        if os.path.isdir(self.values[self.event_key_js['-DIR-']]):
+            self.scan_results = self.scan_directory(directory, self.scan_mode)
+            self.browse_update(key=self.event_key_js['-BROWSER_LIST-'],args={"values":self.scan_results})
     @staticmethod
     def get_scan_browser_layout(section=None,extra_buttons=[]):
         """
@@ -66,12 +71,16 @@ class AbstractBrowser:
         return [
         [make_component("Text",'Directory to scan:'), make_component("InputText",os.getcwd(),key=text_to_key(text='dir',section=section)),
          make_component("FolderBrowse","Folders", enable_events=True, key=text_to_key(text='directory browser',section=section)),
-         make_component("FilesBrowse","Files", enable_events=True, key=text_to_key(text='file browser',section=section))],
-        [make_component("Listbox",[],size=(50, 10),key=text_to_key(text='files list',section=section),enable_events=True),make_component("Checkbox",'all',default=True,key=text_to_key(text='scan mode all',section=section))],
+         make_component("FilesBrowse","Files", enable_events=True, key=text_to_key(text='file browser',section=section)),
+         make_component("Checkbox",'all directory',default=True,key=text_to_key(text='scan mode all',section=section))], 
+        [make_component("Listbox",[],size=(50, 10),key=text_to_key(text='browser list',section=section),enable_events=True),
+         make_component("Listbox",[],size=(50, 10),key=text_to_key(text='files list',section=section),enable_events=True)],
         [make_component("Button","Scan",key=text_to_key(text='scan',section=section),enable_events=True),
         make_component("Button","<-",key=text_to_key(text='browse backwards',section=section),enable_events=True),
         make_component("Button","All Scan Mode",key=text_to_key(text='mode',section=section),enable_events=True),
-        make_component("Button","->",key=text_to_key(text='browse forwards',section=section),enable_events=True)]+extra_buttons
+        make_component("Button","->",key=text_to_key(text='browse forwards',section=section),enable_events=True),
+         make_component("Button","Add",key=text_to_key(text='add_to_list',section=section),enable_events=True),
+         make_component("Button","Clear",key=text_to_key(text='clear list',section=section),enable_events=True)]+extra_buttons
         ]
     def while_static(self,window,event_key_js,values):
         self.values,self.window=values,window
@@ -82,33 +91,48 @@ class AbstractBrowser:
         if self.event_key_js['found'] == '-SCAN-':
             self.scan_it(self.return_directory())
         if self.event_key_js['found'] == "-SELECT_HIGHLIGHTED-":
-            if len(self.values[self.event_key_js['-FILES_LIST-']])>0:
-                self.browse_update(key=self.event_key_js['-DIR-'],args={"value":os.path.join(self.return_directory(), self.values[self.event_key_js['-FILES_LIST-']][0])})
+            if len(self.values[self.event_key_js['-BROWSER_LIST-']])>0:
+                self.browse_update(key=self.event_key_js['-DIR-'],args={"value":os.path.join(self.return_directory(), self.values[self.event_key_js['-BROWSER_LIST-']][0])})
         if self.event_key_js['found'] == '-MODE-':
             self.scan_mode = 'folder' if self.scan_mode == 'file' else 'file'
             if self.values['-SCAN_MODE_ALL-']:
                 self.scan_mode='all'
             self.window.Element(self.event_key_js['-MODE-']).update(text=f"{self.scan_mode[0].upper()}{self.scan_mode[1:]}  Scan Mode")
-
+        if self.event_key_js['found'] == '-ADD_TO_LIST-':
+            file_list = self.values[self.event_key_js['-BROWSER_LIST-']]
+            files_list = self.values[self.event_key_js['-FILES_LIST-']] or []
+            if file_list:
+                files_list.append(file_list[0])
+            path_dir = self.values[self.event_key_js['-DIR-']].split(';')
+            for path in path_dir:
+                if path not in files_list:
+                    files_list.append(path)
+            self.window[self.event_key_js['-FILES_LIST-']].update(files_list)
+        if self.event_key_js['found'] == '-CLEAR_LIST-':
+            self.window[self.event_key_js['-FILES_LIST-']].update([])
         if self.event_key_js['found'] == "-BROWSE_BACKWARDS-":
             # Navigate up to the parent directory
             if self.return_directory() not in self.history:
                 self.history.append(self.return_directory())
             directory = os.path.dirname(self.return_directory())  # This will give the parent directory
             self.browse_update(key=self.event_key_js['-DIR-'],args={"value":directory})
-            self.browse_update(key=self.event_key_js['-FILES_LIST-'],args={"values":self.scan_directory(directory, self.scan_mode)})
+            self.browse_update(key=self.event_key_js['-BROWSER_LIST-'],args={"values":self.scan_directory(directory, self.scan_mode)})
             self.scan_it(directory)
-        if self.event_key_js['found'] in ["-BROWSE_FORWARDS-",'-FILES_LIST-']:
-            # Navigate down into the selected directory or move to the next history path
-            if self.values[self.event_key_js['-FILES_LIST-']]:  # If there's a selected folder in the listbox
-                directory = os.path.join(self.return_directory(), self.values[self.event_key_js['-FILES_LIST-']][0])
-                if os.path.isdir(directory):
-                    self.forward_dir(directory)
-                    self.scan_it(directory)
-            elif self.history:  # If there's a directory in the history stack
-                directory = self.history.pop()
-                self.browse_update(key=self.event_key_js['-DIR-'],args={"value":directory})
-                self.browse_update(key=self.event_key_js['-FILES_LIST-'],args={"values":self.scan_directory(directory, self.scan_mode)})
+        if self.event_key_js['found'] in ["-BROWSE_FORWARDS-",'-BROWSER_LIST-']:
+            directory=None
+            try:
+                # Navigate down into the selected directory or move to the next history path
+                if self.values[self.event_key_js['-BROWSER_LIST-']]:  # If there's a selected folder in the listbox
+                    directory = os.path.join(self.return_directory(), self.values[self.event_key_js['-BROWSER_LIST-']][0])
+                    if os.path.isdir(directory):
+                        self.forward_dir(directory)
+                        self.scan_it(directory)
+                elif self.history:  # If there's a directory in the history stack
+                    directory = self.history.pop()
+                    self.browse_update(key=self.event_key_js['-DIR-'],args={"value":directory})
+                    self.browse_update(key=self.event_key_js['-BROWSER_LIST-'],args={"values":self.scan_directory(directory, self.scan_mode)})
+            except:
+                print(f'could not scan directory {directory}')
         return self.event,self.values
     def return_directory(self):
         """
@@ -177,7 +201,7 @@ class AbstractBrowser:
         """
         if os.path.isdir(directory):
             self.browse_update(key=self.event_key_js['-DIR-'],args={"value":directory})
-            self.browse_update(key=self.event_key_js['-FILES_LIST-'],args={"values":self.scan_directory(directory, self.scan_mode)})
+            self.browse_update(key=self.event_key_js['-BROWSER_LIST-'],args={"values":self.scan_directory(directory, self.scan_mode)})
     def scan_directory(self,directory_path, mode):
         """
         List files or folders in the given directory based on the provided mode.
